import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:math';

final gz1 = new GZipCodec(level: 1);
final gz3 = new GZipCodec(level: 3);
final gz5 = new GZipCodec(level: 5);
final gz6 = new GZipCodec(level: 6);
final gz7 = new GZipCodec(level: 7);
final gz9 = new GZipCodec(level: 9);

final random = new Random.secure();
final mb = 128 * 1024;

main() async {
  final textBuf = new File('test/golden/wiki-lzma.html').readAsBytesSync(); //randomText(mb);
  final binBuf = randomBin(mb);

  print('text');
  for (Codec codec in [gz1, gz3, gz5, gz6, gz7, gz9]) {
    benchCodec(codec, textBuf);
  }
  await benchGzip(textBuf);

  print('bin');
  for (Codec codec in [gz1, gz3, gz5, gz7, gz9]) {
    benchCodec(codec, binBuf);
  }
  await benchGzip(binBuf);
}

void benchCodec(Codec<List<int>, List<int>> codec, List<int> input) {
  Stopwatch sw1 = new Stopwatch()..start();
  List<int> encoded;
  for (int i = 0; i < 100; i++) {
    encoded = codec.encode(input);
  }
  sw1.stop();
  Stopwatch sw2 = new Stopwatch()..start();
  for (int i = 0; i < 100; i++) {
    codec.decode(encoded);
  }
  sw2.stop;
  print([
    (encoded.length * 100 / input.length).toStringAsFixed(2),
    sw1.elapsedMilliseconds,
    sw2.elapsedMilliseconds,
  ]);
}


Future benchGzip(List<int> input) async {
  Stopwatch sw1 = new Stopwatch()..start();
  List<int> encoded;
  for (int i = 0; i < 100; i++) {
    encoded = await cmdGzip(input);
  }
  sw1.stop();
  Stopwatch sw2 = new Stopwatch()..start();
  for (int i = 0; i < 100; i++) {
    gz1.decode(encoded);
  }
  sw2.stop;
  print(['cmd',
    (encoded.length * 100 / input.length).toStringAsFixed(2),
    sw1.elapsedMilliseconds,
    sw2.elapsedMilliseconds,
  ]);
}

List<int> randomText(int length) {
  return new List.generate(
      length,
      (_) => random.nextInt(random.nextInt(
              random.nextInt(random.nextInt(random.nextInt(160) + 1) + 1) + 1) +
          1));
}

List<int> randomBin(int length) {
  return new List.generate(length, (_) => random.nextInt(224));
}

Future<List<int>> cmdGzip(List<int> data) async {
  final p = await Process.start('gzip', ['-c']);
  p.stdin.add(data);
  await p.stdin.close();
  return await p.stdout.fold([], (a ,b)=>a);
}
